import h5py
import numpy as np
import read_solar as rs
import matplotlib.pyplot as plt

# Change these values to point to the product files
hdf5_file_path = "g3b.ssp.2021011831SSv05.30"
binary_file_path = "g3b.sspb.2021011831SSv05.30"

##############################################################################
# Plot an ozone profile (altitude vs. concentration) using
# a formatted (HDF5) product file
# Requires the h5py package

# Open the HDF5 file
file_id = h5py.File(hdf5_file_path, 'r')

# Isolate the dataset of interest
ozone_profile = file_id['/Altitude Based Data/Aerosol Ozone Profiles/Ozone_AO3']

# Read the fill value from the dataset attributes
ozone_fill_value = ozone_profile.attrs['_FillValue']

# For plotting purposes, let's also grab altitudes and event ID
altitudes = file_id['/Altitude Based Data/Altitude Information/Altitude']
event_id = file_id['/Event Information/Event_ID'][()].decode()

# Establish a mask for fill values
mask = ozone_profile < ozone_fill_value

# Plot the ozone profile
plt.plot(ozone_profile[mask], altitudes[mask])
plt.title("Event " + str(event_id))
plt.xlabel("Ozone Concentration (cm^-3)")
plt.ylabel("Altitude (km)")
plt.xlim(0, 1e13)
plt.savefig("ozone_"+ str(event_id) + ".png")
plt.close()

##############################################################################
# Plot an aerosol profile (altitude vs. extinction coefficient) using
# an unformatted (binary) product file
# Requires read_solar.py

# Read the binary product file
data,stat =  rs.sol_l2_bin(binary_file_path)
event_id = data.event_id

# Aerosol extinction coefficient profiles are given for 9 channels,
# lets use the data from the next-to-last (wavelength ~1 micron)
aerosol_profile = data.AerosolExt.aerext[:,-2]

# Read the fill value for float variables
aerosol_fill_value = data.flt_fill_value

# For plotting purposes, let's also grab the altitudes (aerosol only
# uses the first 90 of these), event ID, and aerosol wavelength
altitudes = data.Altitude[:90]
wavelength = data.Aer_wavelength[-2]

# Establish a mask for fill values
mask = aerosol_profile < aerosol_fill_value

# Plot the aerosol extinction profile
plt.plot(aerosol_profile[mask], altitudes[mask])
plt.title("Event " + event_id)
plt.xlabel("{0:.1f}".format(wavelength) + " nm Aerosol Extinction Coefficient (km^-1)")
plt.ylabel("Altitude (km)")
plt.xscale('log')
plt.xlim(1e-6, 1e-2)
plt.savefig("aerosol_" + event_id + ".png")
plt.close()
