pro read_solar, filename, status, solar

;Disclaimers
;No Warranty: THE SUBJECT SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY OF
;ANY KIND, EITHER EXPRESSED, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT LIMITED
;TO, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL CONFORM TO SPECIFICATIONS,
;ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
;OR FREEDOM FROM INFRINGEMENT, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL BE
;ERROR FREE, OR ANY WARRANTY THAT DOCUMENTATION, IF PROVIDED, WILL CONFORM TO
;THE SUBJECT SOFTWARE. THIS AGREEMENT DOES NOT, IN ANY MANNER, CONSTITUTE AN
;ENDORSEMENT BY GOVERNMENT AGENCY OR ANY PRIOR RECIPIENT OF ANY RESULTS,
;RESULTING DESIGNS, HARDWARE, SOFTWARE PRODUCTS OR ANY OTHER APPLICATIONS
;RESULTING FROM USE OF THE SUBJECT SOFTWARE.  FURTHER, GOVERNMENT AGENCY
;DISCLAIMS ALL WARRANTIES AND LIABILITIES REGARDING THIRD-PARTY SOFTWARE, IF
;PRESENT IN THE ORIGINAL SOFTWARE, AND DISTRIBUTES IT "AS IS." 
 
;Waiver and Indemnity:  RECIPIENT AGREES TO WAIVE ANY AND ALL CLAIMS AGAINST
;THE UNITED STATES GOVERNMENT, ITS CONTRACTORS AND SUBCONTRACTORS, AS WELL AS
;ANY PRIOR RECIPIENT.  IF RECIPIENT'S USE OF THE SUBJECT SOFTWARE RESULTS IN
;ANY LIABILITIES, DEMANDS, DAMAGES, EXPENSES OR LOSSES ARISING FROM SUCH USE,
;INCLUDING ANY DAMAGES FROM PRODUCTS BASED ON, OR RESULTING FROM, RECIPIENT'S
;USE OF THE SUBJECT SOFTWARE, RECIPIENT SHALL INDEMNIFY AND HOLD HARMLESS THE
;UNITED STATES GOVERNMENT, ITS CONTRACTORS AND SUBCONTRACTORS, AS WELL AS ANY
;PRIOR RECIPIENT, TO THE EXTENT PERMITTED BY LAW.  RECIPIENT'S SOLE REMEDY FOR
;ANY SUCH MATTER SHALL BE THE IMMEDIATE, UNILATERAL TERMINATION OF THIS
;AGREEMENT.

;  Purpose:
;  This program provides the data structures necessary to read the
;  SAGE III Solar Event Species Data binary product files.
;
;  Authors:
;  Mike Heitz <michael.l.heitz@nasa.gov>
;  Carter Hulsey
;  Robbie Manion
;  Marsha Larosee
;  David Huber
;  Karen Sage
;  
;  Revision History:
;  11/01/2000 - Initial Coding.
;  08/01/01 - Deleted variable definitions and read of pressure
;             surface variables because of change in file format.
;  10/16/02 - Redesigned code to return same sized solar array for 
;             any version of product files.
;  02/25/03 - Added check of filename for string 'sspb' to relieve
;             problems if the directory contained a 'v'.
;  03/12/03 - Renamed Director Version and Solar Version to 
;             Software Version and Dataproduct Version.
;  04/21/03 - Changed to select software version from first read of 
;             solar_size structure.
;  04/21/03 - Placed vers selection after swap_endian so that it
;             is correct on all machines.
;  09/19/03 - Added new variable (EvtStatusFlag) to solar structure.  
;             Made code backward compatible.
;  11/18/03 - Removed diagnostic statement.
;  01/31/04 - Added code for versioning the reader.
;              **** Any time product format changes are made from this
;              **** time forward, the save_solar_reader_version 
;              **** variable must be incremented.
;  09/19/18 - Added compatability with version 5.1

;  Input variable:  the name of the solar product file (the product 
;     file names have the form  g3a.sspb.*)
;  Output variables:  a read file status variable and the structure 
;     containing the solar product data.  See the SAGE III Data
;     Products Users Guide for a description of the solar product data.

;  Initialize local variables.

status = 'OK'
IEEE_flag = 0

itest = 512
ltest = 102580L
ftest = 1.23e10
dtest = 1.23d50

save_solar_reader_version =  0        

;  Determine if the current machine uses IEEE, big_endian numbers.
;    A sample int, long, float and double are converted using
;    byteorder and compared with the original.  If there is no 
;    change, the machine is assumed to be IEEE compliant and big-endian.
;    

it2 = itest
lt2 = ltest
ft2 = ftest
dt2 = dtest

byteorder, it2, /htons
byteorder, lt2, /htonl
byteorder, ft2, /ftoxdr
byteorder, dt2, /dtoxdr

case 1 of
(itest eq it2) and (ltest eq lt2) and (ftest eq ft2) and (dtest eq dt2) : IEEE_flag = 1
else:  IEEE_flag = 0
endcase

;Get Version from file name 
splt_file = STRSPLIT(filename, 'v', /EXTRACT)
vers = FLOAT(splt_file[-1])

;  Create the structure for read from binary file of the data needed
;  to dynamically size data arrays.
IF vers GE 5.3 THEN BEGIN
        solar_size = {event_id:'XXXXXXXXXXXX', $
                         old_event_id:0L, $
                         Date:0L, $
                         fraction_time:0.D, $
                         Latitude:0L, $
                         Longitude:0L, $
                         Time:0L, $
                         int_fill_value:0L, $
                         flt_fill_value:0., $
                         mission_id:0L, $
                         L0DO_Version:0., $
                         CCDVersion:0., $
                         L0_Version:0., $
                         Software_Version:0., $
                         Dataproduct_Version:0., $
                         Spectroscopic_DataBase_Version:0., $
                         GRAM95_Version:0., $
                         Met_Version:0., $
                         bin_height:0., $
                         num_bins:0L, $
                         num_met_grid:0L, $
                         num_aer_channels:0L,$
                         num_grnd_trk:0L, $
                         num_aer_bins:0L}

ENDIF ELSE IF vers GE 5.2 THEN BEGIN
        solar_size = {event_id:'XXXXXXXXXXXX', $
                         old_event_id:0L, $
                         Date:0L, $
                         fraction_time:0., $
                         Latitude:0L, $
                         Longitude:0L, $
                         Time:0L, $
                         int_fill_value:0L, $
                         flt_fill_value:0., $
                         mission_id:0L, $
                         L0DO_Version:0., $
                         CCDVersion:0., $
                         L0_Version:0., $
                         Software_Version:0., $
                         Dataproduct_Version:0., $
                         Spectroscopic_DataBase_Version:0., $
                         GRAM95_Version:0., $
                         Met_Version:0., $
                         bin_height:0., $
                         num_bins:0L, $
                         num_met_grid:0L, $
                         num_aer_channels:0L,$
                         num_grnd_trk:0L, $
                         num_aer_bins:0L}


ENDIF ELSE IF vers EQ 5.1 THEN BEGIN
	solar_size = {event_id:0L, $
                         Date:0L, $
                         fraction_time:0., $
                         Latitude:0L, $
                         Longitude:0L, $
                         Time:0L, $
                         int_fill_value:0L, $
                         flt_fill_value:0., $
                         mission_id:0L, $
                         L0DO_Version:0., $
                         CCDVersion:0., $
                         L0_Version:0., $
                         Software_Version:0., $
                         Dataproduct_Version:0., $
                         Spectroscopic_DataBase_Version:0., $
                         GRAM95_Version:0., $
                         Met_Version:0., $
                         bin_height:0., $
                         num_bins:0L, $
                         num_met_grid:0L, $
                         num_aer_channels:0L,$
                         num_grnd_trk:0L, $
                         num_aer_bins:0L}

ENDIF ELSE BEGIN
	solar_size = {event_id:0L, $
	                 yr_day_tag:0L, $
	                 mission_time:0., $
	                 int_fill_value:0L, $
	                 flt_fill_value:0., $
	                 mission_id:0L, $
	                 L0DO_Version:0., $
	                 L0_Version:0., $
	                 Software_Version:0., $
	      	         Dataproduct_Version:0., $
	                 DB_Version:0., $
	                 GRAM95_Version:0., $
	                 Met_Version:0., $
	                 bin_height:0., $
	                 num_bins:0L, $
	                 num_ao3_channels:0L,$
	                 num_grnd_trk:0L, $
	                 num_aer_bins:0L}
ENDELSE
;  Extract software version from version number is this structure.

on_ioerror, open_error
openr, in_unit, filename, /get_lun

;  Read size structure.
readu, in_unit, solar_size

;  Open file for direct read-only access.

;  If machine is not IEEE then swap bytes to make them compliant.
 
if IEEE_flag eq 0 then solar_size = swap_endian (solar_size)

;  Create a structure for the aerosol extinction channel data.
;  NOTE:  The tag AerosolExt is an array containing this structure
;         for each channel.  To reference the i-th element of the
;         AerosolExt, the correct syntax is solar.AerosolExt(i).aerext

aerosol_ext = { aerosol_ext, $
                aerext:fltarr(solar_size.num_aer_bins), $
                aerext_uncert:fltarr(solar_size.num_aer_bins), $
                aerQA:lonarr(solar_size.num_aer_bins) }

IF vers GE 5.3 THEN BEGIN
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;  Define the solar structure for version 5.3 ;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  solar = { event_id:'XXXXXXXXXXXX', $
      old_event_id:0L, $
      Date:0L, $
      Year_Fraction:0.D, $
      Latitude:0., $
      Longitude:0., $
      Time:0L, $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      CCDTable_Version:0L, $
      L0_Version:0., $
      Software_Version:0., $
      Dataproduct_Version:0., $
      Spectroscopic_Database_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      bin_height:0., $
      num_bins:0L, $
      num_met_grid:0L, $
      num_aer_channels:0L,$
      num_grnd_trk:0L, $
      num_aer_bins:0L, $
      sc_evt_type:0L, $
      gnd_evt_type:0L, $
      BetaAngle_Solar:0., $
      Aurora_Flag:0L, $
      Ephemeris_Source:0L, $
      gt_date:lonarr(solar_size.num_grnd_trk), $
      gt_time:lonarr(solar_size.num_grnd_trk), $
      gt_latitude:fltarr(solar_size.num_grnd_trk), $
      gt_longitude:fltarr(solar_size.num_grnd_trk), $
      gt_ray_dir:fltarr(solar_size.num_grnd_trk), $
      Space_Craft_Lat:fltarr(solar_size.num_grnd_trk), $
      Space_Craft_Lon:fltarr(solar_size.num_grnd_trk), $
      Space_Craft_Alt:fltarr(solar_size.num_grnd_trk), $
      Homogeneity:lonarr(solar_size.num_bins), $
      Altitude:fltarr(solar_size.num_bins), $
      Geopotential_Alt:fltarr(solar_size.num_bins), $
      Temperature:fltarr(solar_size.num_bins), $
      Temperature_uncert:fltarr(solar_size.num_bins), $
      Pressure:fltarr(solar_size.num_bins), $
      Pressure_uncert:fltarr(solar_size.num_bins), $
      Neutral_Density:fltarr(solar_size.num_bins), $
      Neutral_Density_uncert:fltarr(solar_size.num_bins), $
      Temp_Pressure_Source:lonarr(solar_size.num_bins), $
      trop_temp:0., $
      trop_alt:0., $
      trop_press:0., $
      met_pressure:fltarr(solar_size.num_met_grid), $
      met_temp:fltarr(solar_size.num_met_grid), $
      met_temp_unc:fltarr(solar_size.num_met_grid), $
      met_altitude:fltarr(solar_size.num_met_grid), $
      met_Source:0L, $
      CCD_Temperature:0., $
      Spectrometer_Zenith_Temperature:0., $
      CCD_Temperature_minus_TEC:0., $
      Ephemeris_Quality:0L, $
      SpecCalShift:0., $
      SpecCalStretch:0., $
      AzimuthAngle:fltarr(2), $
      QAFlag:0L, $
      QAFlag_Altitude:lonarr(solar_size.num_bins), $
      Ozone_Composite:fltarr(solar_size.num_bins), $
      Ozone_Composite_uncert:fltarr(solar_size.num_bins), $
      Ozone_Composite_QA:lonarr(solar_size.num_bins), $
      Ozone_Mes:fltarr(solar_size.num_bins), $
      Ozone_Mes_uncert:fltarr(solar_size.num_bins), $
      Ozone_Mes_QA:lonarr(solar_size.num_bins), $
      Ozone_MLR:fltarr(solar_size.num_bins), $
      Ozone_MLR_uncert:fltarr(solar_size.num_bins), $
      Ozone_MLR_QA:lonarr(solar_size.num_bins), $
      Ozone_aO3:fltarr(solar_size.num_bins), $
      Ozone_aO3_uncert:fltarr(solar_size.num_bins), $
      Ozone_aO3_QA:lonarr(solar_size.num_bins), $
      H2O:fltarr(solar_size.num_bins), $
      H2O_uncert:fltarr(solar_size.num_bins), $
      H2O_QA:lonarr(solar_size.num_bins), $
      NO2:fltarr(solar_size.num_bins), $
      NO2_uncert:fltarr(solar_size.num_bins), $
      NO2_QA:lonarr(solar_size.num_bins), $
      RetTemp:fltarr(solar_size.num_bins), $
      RetTemp_uncert:fltarr(solar_size.num_bins), $
      RetPress:fltarr(solar_size.num_bins), $
      RetPress_uncert:fltarr(solar_size.num_bins), $
      RetTP_QA:lonarr(solar_size.num_bins), $
      Aer_wavelength:fltarr(solar_size.num_aer_channels), $
      Aer_width:fltarr(solar_size.num_aer_channels), $
      Molecular_SCT:fltarr(solar_size.num_aer_channels), $
      Molecular_SCT_uncert:fltarr(solar_size.num_aer_channels),$
      Strat_Aer_OD:fltarr(solar_size.num_aer_channels), $
      Strat_Aer_OD_uncert:fltarr(solar_size.num_aer_channels), $
      Strat_Aer_OD_QA:lonarr(solar_size.num_aer_channels), $
      AerosolExt:replicate(aerosol_ext, solar_size.num_aer_channels)}

   point_lun, in_unit, 0
   readu, in_unit, solar
   if IEEE_flag eq 0 then solar = swap_endian(solar)

   ;Add Separate Flags for Event QA BitFlag Entries, IDL 1/0 = True/False
   solar_addtags={HexErrFlag:0L,ContWindowClosedFlag:0L,TimeQualFlag:0L,DMPExoFlag:0L,BlockExoFlag:0L,$
         SpectralCalFlag:0L,SolarEclipseFlag:0L,OffNadirFlag:0L,DMPSkipFlag:0L,DMPAltFlag:lonarr(solar_size.num_bins)}

   solar=create_struct(solar,solar_addtags)

   if ((solar.QAFlag and 2L^0) ne 0) then solar.HexErrFlag=1 else solar.HexErrFlag=0
   if ((solar.QAFlag and 2L^1) ne 0) then solar.ContWindowClosedFlag=1 else solar.ContWindowClosedFlag=0
   if ((solar.QAFlag and 2L^2) ne 0) then solar.TimeQualFlag=1 else solar.TimeQualFlag=0
   if ((solar.QAFlag and 2L^3) ne 0) then solar.DMPExoFlag=1 else solar.DMPExoFlag=0
   if ((solar.QaFlag and 2L^4) ne 0) then solar.BlockExoFlag=1 else solar.BlockExoFlag=0
   if ((solar.QAFlag and 2L^5) ne 0) then solar.SpectralCalFlag=1 else solar.SpectralCalFlag=0
   if ((solar.QAFlag and 2L^6) ne 0) then solar.SolarEclipseFlag=1 else solar.SolarEclipseFlag=0
   if ((solar.QAFlag and 2L^7) ne 0) then solar.OffNadirFlag=1 else solar.OffNadirFlag=0
   if ((solar.QAFlag and 2L^8) ne 0) then solar.DMPSkipFlag=1 else solar.DMPSkipFlag=0

   ;Altitude Based Bit Flags
   for i = 0,solar_size.num_bins-1 do begin
      if ((solar.QAFlag_Altitude(i) and 2L^0) ne 0) then solar.DMPAltFlag(i)=1 else solar.DMPAltFlag(i)=0
   endfor

   save_reader_temp =  solar.Ephemeris_Source
   save_reader_templ =  ishft(save_reader_temp,24)
   solar_reader_version =  ishft(save_reader_templ,-27)
   close, in_unit
   free_lun, in_unit
   goto, program_end

ENDIF ELSE IF vers GE 5.2 THEN BEGIN
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;  Define the solar structure for version 5.2 ;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  solar = { event_id:'XXXXXXXXXXXX', $
      old_event_id:0L, $
      Date:0L, $
      Year_Fraction:0., $
      Latitude:0., $
      Longitude:0., $
      Time:0L, $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      CCDTable_Version:0L, $
      L0_Version:0., $
      Software_Version:0., $
      Dataproduct_Version:0., $
      Spectroscopic_Database_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      bin_height:0., $
      num_bins:0L, $
      num_met_grid:0L, $
      num_aer_channels:0L,$
      num_grnd_trk:0L, $
      num_aer_bins:0L, $
      sc_evt_type:0L, $
      gnd_evt_type:0L, $
      BetaAngle_Solar:0., $
      Aurora_Flag:0L, $
      Ephemeris_Source:0L, $
      gt_date:lonarr(solar_size.num_grnd_trk), $
      gt_time:lonarr(solar_size.num_grnd_trk), $
      gt_latitude:fltarr(solar_size.num_grnd_trk), $
      gt_longitude:fltarr(solar_size.num_grnd_trk), $
      gt_ray_dir:fltarr(solar_size.num_grnd_trk), $
      Space_Craft_Lat:fltarr(solar_size.num_grnd_trk), $
      Space_Craft_Lon:fltarr(solar_size.num_grnd_trk), $
      Space_Craft_Alt:fltarr(solar_size.num_grnd_trk), $
      Homogeneity:lonarr(solar_size.num_bins), $
      Altitude:fltarr(solar_size.num_bins), $
      Geopotential_Alt:fltarr(solar_size.num_bins), $
      Temperature:fltarr(solar_size.num_bins), $
      Temperature_uncert:fltarr(solar_size.num_bins), $
      Pressure:fltarr(solar_size.num_bins), $
      Pressure_uncert:fltarr(solar_size.num_bins), $
      Neutral_Density:fltarr(solar_size.num_bins), $
      Neutral_Density_uncert:fltarr(solar_size.num_bins), $
      Temp_Pressure_Source:lonarr(solar_size.num_bins), $
      trop_temp:0., $
      trop_alt:0., $
      trop_press:0., $
      met_pressure:fltarr(solar_size.num_met_grid), $
      met_temp:fltarr(solar_size.num_met_grid), $
      met_temp_unc:fltarr(solar_size.num_met_grid), $
      met_altitude:fltarr(solar_size.num_met_grid), $
      met_Source:0L, $
      CCD_Temperature:0., $
      Spectrometer_Zenith_Temperature:0., $
      CCD_Temperature_minus_TEC:0., $
      Ephemeris_Quality:0L, $
      SpecCalShift:0., $
      SpecCalStretch:0., $
      AzimuthAngle:fltarr(2), $
      QAFlag:0L, $
      QAFlag_Altitude:lonarr(solar_size.num_bins), $
      Ozone_Composite:fltarr(solar_size.num_bins), $
      Ozone_Composite_uncert:fltarr(solar_size.num_bins), $
      Ozone_Composite_QA:lonarr(solar_size.num_bins), $
      Ozone_Mes:fltarr(solar_size.num_bins), $
      Ozone_Mes_uncert:fltarr(solar_size.num_bins), $
      Ozone_Mes_QA:lonarr(solar_size.num_bins), $
      Ozone_MLR:fltarr(solar_size.num_bins), $
      Ozone_MLR_uncert:fltarr(solar_size.num_bins), $
      Ozone_MLR_QA:lonarr(solar_size.num_bins), $
      Ozone_aO3:fltarr(solar_size.num_bins), $
      Ozone_aO3_uncert:fltarr(solar_size.num_bins), $
      Ozone_aO3_QA:lonarr(solar_size.num_bins), $
      H2O:fltarr(solar_size.num_bins), $
      H2O_uncert:fltarr(solar_size.num_bins), $
      H2O_QA:lonarr(solar_size.num_bins), $
      NO2:fltarr(solar_size.num_bins), $
      NO2_uncert:fltarr(solar_size.num_bins), $
      NO2_QA:lonarr(solar_size.num_bins), $
      RetTemp:fltarr(solar_size.num_bins), $
      RetTemp_uncert:fltarr(solar_size.num_bins), $
      RetPress:fltarr(solar_size.num_bins), $
      RetPress_uncert:fltarr(solar_size.num_bins), $
      RetTP_QA:lonarr(solar_size.num_bins), $
      Aer_wavelength:fltarr(solar_size.num_aer_channels), $
      Aer_width:fltarr(solar_size.num_aer_channels), $
      Molecular_SCT:fltarr(solar_size.num_aer_channels), $
      Molecular_SCT_uncert:fltarr(solar_size.num_aer_channels),$
      Strat_Aer_OD:fltarr(solar_size.num_aer_channels), $
      Strat_Aer_OD_uncert:fltarr(solar_size.num_aer_channels), $
      Strat_Aer_OD_QA:lonarr(solar_size.num_aer_channels), $
      AerosolExt:replicate(aerosol_ext, solar_size.num_aer_channels)}

   point_lun, in_unit, 0
   readu, in_unit, solar
   if IEEE_flag eq 0 then solar = swap_endian(solar)

   ;Add Separate Flags for Event QA BitFlag Entries, IDL 1/0 = True/False
   solar_addtags={HexErrFlag:0L,ContWindowClosedFlag:0L,TimeQualFlag:0L,DMPExoFlag:0L,BlockExoFlag:0L,$
         SpectralCalFlag:0L,SolarEclipseFlag:0L,DMPAltFlag:lonarr(solar_size.num_bins)}

   solar=create_struct(solar,solar_addtags)

   if ((solar.QAFlag and 2L^0) ne 0) then solar.HexErrFlag=1 else solar.HexErrFlag=0
   if ((solar.QAFlag and 2L^1) ne 0) then solar.ContWindowClosedFlag=1 else solar.ContWindowClosedFlag=0
   if ((solar.QAFlag and 2L^2) ne 0) then solar.TimeQualFlag=1 else solar.TimeQualFlag=0
   if ((solar.QAFlag and 2L^3) ne 0) then solar.DMPExoFlag=1 else solar.DMPExoFlag=0
   if ((solar.QaFlag and 2L^4) ne 0) then solar.BlockExoFlag=1 else solar.BlockExoFlag=0
   if ((solar.QAFlag and 2L^5) ne 0) then solar.SpectralCalFlag=1 else solar.SpectralCalFlag=0
   if ((solar.QAFlag and 2L^6) ne 0) then solar.SolarEclipseFlag=1 else solar.SolarEclipseFlag=0

   ;Altitude Based Bit Flags
   for i = 0,solar_size.num_bins-1 do begin
      if ((solar.QAFlag_Altitude(i) and 2L^0) ne 0) then solar.DMPAltFlag(i)=1 else solar.DMPAltFlag(i)=0
   endfor

   save_reader_temp =  solar.Ephemeris_Source
   save_reader_templ =  ishft(save_reader_temp,24)
   solar_reader_version =  ishft(save_reader_templ,-27)
   close, in_unit
   free_lun, in_unit
   goto, program_end


ENDIF ELSE IF vers EQ 5.1 THEN BEGIN
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;  Define the solar structure for version 5.1 ;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  solar = { event_id:0L, $
      Date:0L, $
      Year_Fraction:0., $
      Latitude:0., $
      Longitude:0., $
      Time:0L, $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      CCDTable_Version:0L, $
      L0_Version:0., $
      Software_Version:0., $
      Dataproduct_Version:0., $
      Spectroscopic_Database_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      bin_height:0., $
      num_bins:0L, $
      num_met_grid:0L, $
      num_aer_channels:0L,$
      num_grnd_trk:0L, $
      num_aer_bins:0L, $
      sc_evt_type:0L, $
      gnd_evt_type:0L, $
      BetaAngle_Solar:0., $
      Aurora_Flag:0L, $
      Ephemeris_Source:0L, $
      gt_date:lonarr(solar_size.num_grnd_trk), $
      gt_time:lonarr(solar_size.num_grnd_trk), $
      gt_latitude:fltarr(solar_size.num_grnd_trk), $
      gt_longitude:fltarr(solar_size.num_grnd_trk), $
      gt_ray_dir:fltarr(solar_size.num_grnd_trk), $
      Space_Craft_Lat:fltarr(solar_size.num_grnd_trk), $
      Space_Craft_Lon:fltarr(solar_size.num_grnd_trk), $
      Space_Craft_Alt:fltarr(solar_size.num_grnd_trk), $
      Homogeneity:lonarr(solar_size.num_bins), $
      Altitude:fltarr(solar_size.num_bins), $
      Geopotential_Alt:fltarr(solar_size.num_bins), $
      Temperature:fltarr(solar_size.num_bins), $
      Temperature_uncert:fltarr(solar_size.num_bins), $
      Pressure:fltarr(solar_size.num_bins), $
      Pressure_uncert:fltarr(solar_size.num_bins), $
      Neutral_Density:fltarr(solar_size.num_bins), $
      Neutral_Density_uncert:fltarr(solar_size.num_bins), $
      Temp_Pressure_Source:lonarr(solar_size.num_bins), $
      trop_temp:0., $
      trop_alt:0., $
      trop_press:0., $
      met_pressure:fltarr(solar_size.num_met_grid), $
      met_temp:fltarr(solar_size.num_met_grid), $
      met_temp_unc:fltarr(solar_size.num_met_grid), $
      met_altitude:fltarr(solar_size.num_met_grid), $
      met_Source:0L, $
      CCD_Temperature:0., $
      Spectrometer_Zenith_Temperature:0., $
      CCD_Temperature_minus_TEC:0., $
      Ephemeris_Quality:0L, $
      SpecCalShift:0., $
      SpecCalStretch:0., $
      QAFlag:0L, $
      QAFlag_Altitude:lonarr(solar_size.num_bins), $
      Ozone_Composite:fltarr(solar_size.num_bins), $
      Ozone_Composite_uncert:fltarr(solar_size.num_bins), $
      Ozone_Composite_QA:lonarr(solar_size.num_bins), $
      Ozone_Mes:fltarr(solar_size.num_bins), $
      Ozone_Mes_uncert:fltarr(solar_size.num_bins), $
      Ozone_Mes_QA:lonarr(solar_size.num_bins), $
      Ozone_MLR:fltarr(solar_size.num_bins), $
      Ozone_MLR_uncert:fltarr(solar_size.num_bins), $
      Ozone_MLR_QA:lonarr(solar_size.num_bins), $
      Ozone_aO3:fltarr(solar_size.num_bins), $
      Ozone_aO3_uncert:fltarr(solar_size.num_bins), $
      Ozone_aO3_QA:lonarr(solar_size.num_bins), $
      H2O:fltarr(solar_size.num_bins), $
      H2O_uncert:fltarr(solar_size.num_bins), $
      H2O_QA:lonarr(solar_size.num_bins), $
      NO2:fltarr(solar_size.num_bins), $
      NO2_uncert:fltarr(solar_size.num_bins), $
      NO2_QA:lonarr(solar_size.num_bins), $
      RetTemp:fltarr(solar_size.num_bins), $
      RetTemp_uncert:fltarr(solar_size.num_bins), $
      RetPress:fltarr(solar_size.num_bins), $
      RetPress_uncert:fltarr(solar_size.num_bins), $
      RetTP_QA:lonarr(solar_size.num_bins), $
      Aer_wavelength:fltarr(solar_size.num_aer_channels), $
      Aer_width:fltarr(solar_size.num_aer_channels), $
      Molecular_SCT:fltarr(solar_size.num_aer_channels), $
      Molecular_SCT_uncert:fltarr(solar_size.num_aer_channels),$
      Strat_Aer_OD:fltarr(solar_size.num_aer_channels), $
      Strat_Aer_OD_uncert:fltarr(solar_size.num_aer_channels), $
      Strat_Aer_OD_QA:lonarr(solar_size.num_aer_channels), $
      AerosolExt:replicate(aerosol_ext, solar_size.num_aer_channels)} 
   
   point_lun, in_unit, 0
   readu, in_unit, solar
   if IEEE_flag eq 0 then solar = swap_endian(solar)

   ;Add Separate Flags for Event QA BitFlag Entries, IDL 1/0 = True/False
   solar_addtags={HexErrFlag:0L,ContWindowClosedFlag:0L,TimeQualFlag:0L,DMPExoFlag:0L,BlockExoFlag:0L,$
         SpectralCalFlag:0L,SolarEclipseFlag:0L,DMPAltFlag:lonarr(solar_size.num_bins)}

   solar=create_struct(solar,solar_addtags)

   if ((solar.QAFlag and 2L^0) ne 0) then solar.HexErrFlag=1 else solar.HexErrFlag=0
   if ((solar.QAFlag and 2L^1) ne 0) then solar.ContWindowClosedFlag=1 else solar.ContWindowClosedFlag=0
   if ((solar.QAFlag and 2L^2) ne 0) then solar.TimeQualFlag=1 else solar.TimeQualFlag=0
   if ((solar.QAFlag and 2L^3) ne 0) then solar.DMPExoFlag=1 else solar.DMPExoFlag=0
   if ((solar.QaFlag and 2L^4) ne 0) then solar.BlockExoFlag=1 else solar.BlockExoFlag=0
   if ((solar.QAFlag and 2L^5) ne 0) then solar.SpectralCalFlag=1 else solar.SpectralCalFlag=0
   if ((solar.QAFlag and 2L^6) ne 0) then solar.SolarEclipseFlag=1 else solar.SolarEclipseFlag=0

   ;Altitude Based Bit Flags
   for i = 0,solar_size.num_bins-1 do begin
      if ((solar.QAFlag_Altitude(i) and 2L^0) ne 0) then solar.DMPAltFlag(i)=1 else solar.DMPAltFlag(i)=0
   endfor

   save_reader_temp =  solar.Ephemeris_Source
   save_reader_templ =  ishft(save_reader_temp,24)
   solar_reader_version =  ishft(save_reader_templ,-27)
   close, in_unit
   free_lun, in_unit
   goto, program_end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;; The Following Structures are for reading older dicontinued file version. Current Data Product ;;;;
;;;;;                      version (5.1 and greater) us the structure above                         ;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ENDIF ELSE IF vers GE 3.0 and vers LE 5.0 THEN BEGIN
;  Define the solar structure.
  solar = { event_id:0L, $
      yr_day_tag:0L, $
      mission_time:0., $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      L0_Version:0., $
      Software_Version:0., $
      Dataproduct_Version:0., $
      DB_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      bin_height:0., $
      num_bins:0L, $
      num_ao3_channels:0L,$
      num_grnd_trk:0L, $
      num_aer_bins:0L, $
      sc_evt_type:0L, $
      gnd_evt_type:0L, $
      BetaAngle:0., $
      EvtStatusFlag:0L, $
      st_date:0L, $
      st_time:0L, $
      st_latitude:0., $
      st_longitude:0., $
      st_altitude:0., $
      end_date:0L, $
      end_time:0L, $
      end_latitude:0., $
      end_longitude:0., $
      end_altitude:0., $
      gt_date:lonarr(solar_size.num_grnd_trk), $
      gt_time:lonarr(solar_size.num_grnd_trk), $
      gt_latitude:fltarr(solar_size.num_grnd_trk), $
      gt_longitude:fltarr(solar_size.num_grnd_trk), $
      gt_ray_dir:fltarr(solar_size.num_grnd_trk), $
      Homogeneity:lonarr(solar_size.num_bins), $
      geom_alt:fltarr(solar_size.num_bins), $
      potential_alt:fltarr(solar_size.num_bins), $
      CurrentTemp:fltarr(solar_size.num_bins), $
      CurrentTemp_uncert:fltarr(solar_size.num_bins), $
      CurrentPress:fltarr(solar_size.num_bins), $
      CurrentPress_uncert:fltarr(solar_size.num_bins), $
      TPSource:lonarr(solar_size.num_bins), $
      trop_temp:0., $
      trop_alt:0., $
      CompositeO3:fltarr(solar_size.num_bins), $
      CompositeO3_uncert:fltarr(solar_size.num_bins), $
      SPOzone:fltarr(solar_size.num_bins), $
      SPOzone_uncert:fltarr(solar_size.num_bins), $
      CompO3QA:lonarr(solar_size.num_bins), $
      MesO3:fltarr(solar_size.num_bins), $
      MesO3_uncert:fltarr(solar_size.num_bins), $
      MesSPOzone:fltarr(solar_size.num_bins), $
      MesSPOzone_uncert:fltarr(solar_size.num_bins), $
      MesO3QA:lonarr(solar_size.num_bins), $
      ChapO3:fltarr(solar_size.num_bins), $
      ChapO3_uncert:fltarr(solar_size.num_bins), $ 
      ChapSPOzone:fltarr(solar_size.num_bins), $
      ChapSPOzone_uncert:fltarr(solar_size.num_bins), $
      MLRO3QA:lonarr(solar_size.num_bins), $
      Ozone_aO3:fltarr(solar_size.num_bins), $
      Ozone_aO3_uncert:fltarr(solar_size.num_bins), $
      aO3SPOzone:fltarr(solar_size.num_bins), $
      aO3SPOzone_uncert:fltarr(solar_size.num_bins), $
      Ozone_aO3QA:lonarr(solar_size.num_bins), $
      WaterVaporNDP:fltarr(solar_size.num_bins), $
      WaterVaporNDP_uncert:fltarr(solar_size.num_bins), $
      H2OQA:lonarr(solar_size.num_bins), $
      NO2NDP:fltarr(solar_size.num_bins), $
      NO2NDP_uncert:fltarr(solar_size.num_bins), $
      SPNO2:fltarr(solar_size.num_bins), $
      SPNO2_uncert:fltarr(solar_size.num_bins), $
      NO2QA:lonarr(solar_size.num_bins), $
      RetTemp:fltarr(solar_size.num_bins), $
      RetTemp_uncert:fltarr(solar_size.num_bins), $
      RetPress:fltarr(solar_size.num_bins), $
      RetPress_uncert:fltarr(solar_size.num_bins), $
      RetTPQA:lonarr(solar_size.num_bins), $
      Aer_wavelength:fltarr(solar_size.num_ao3_channels), $
      Aer_width:fltarr(solar_size.num_ao3_channels), $
      StratOD:fltarr(solar_size.num_ao3_channels), $
      StratOD_uncert:fltarr(solar_size.num_ao3_channels), $
      StratODQA:lonarr(solar_size.num_ao3_channels), $
      AerosolExt:replicate(aerosol_ext, solar_size.num_ao3_channels) , $
      asd_flag:lonarr(solar_size.num_aer_bins), $
      Aerosol1020:fltarr(solar_size.num_aer_bins), $
      Aerosol1020_uncert:fltarr(solar_size.num_aer_bins), $
      Aero1020QA:lonarr(solar_size.num_aer_bins) }
ENDIF ELSE if vers le 1.06 then BEGIN
  solar_reader_version =  save_solar_reader_version
  solar_temp = { event_id:0L, $
      yr_day_tag:0L, $
      mission_time:0., $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      L0_Version:0., $
      Software_Version:0., $
      Dataproduct_Version:0., $
      Spectroscopic_DataBase_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      bin_height:0., $
      num_bins:0L, $
      num_ao3_channels:0L,$
      num_grnd_trk:0L, $
      num_aer_bins:0L, $
      sc_evt_type:0L, $
      gnd_evt_type:0L, $
      BetaAngle:0., $
      st_date:0L, $
      st_time:0L, $
      st_latitude:0., $
      st_longitude:0., $
      st_altitude:0., $
      end_date:0L, $
      end_time:0L, $
      end_latitude:0., $
      end_longitude:0., $
      end_altitude:0., $
      gt_date:lonarr(solar_size.num_grnd_trk), $
      gt_time:lonarr(solar_size.num_grnd_trk), $
      gt_latitude:fltarr(solar_size.num_grnd_trk), $
      gt_longitude:fltarr(solar_size.num_grnd_trk), $
      gt_ray_dir:fltarr(solar_size.num_grnd_trk), $
      Homogeneity:lonarr(solar_size.num_bins), $
      geom_alt:fltarr(solar_size.num_bins), $
      potential_alt:fltarr(solar_size.num_bins), $
      CurrentTemp:fltarr(solar_size.num_bins), $
      CurrentTemp_uncert:fltarr(solar_size.num_bins), $
      CurrentPress:fltarr(solar_size.num_bins), $
      CurrentPress_uncert:fltarr(solar_size.num_bins), $
      TPSource:lonarr(solar_size.num_bins), $
      trop_temp:0., $
      trop_alt:0., $
      CompositeO3:fltarr(solar_size.num_bins), $
      CompositeO3_uncert:fltarr(solar_size.num_bins), $
      SPOzone:fltarr(solar_size.num_bins), $
      SPOzone_uncert:fltarr(solar_size.num_bins), $
      CompO3QA:lonarr(solar_size.num_bins), $
      MesO3:fltarr(solar_size.num_bins), $
      MesO3_uncert:fltarr(solar_size.num_bins), $
      MesSPOzone:fltarr(solar_size.num_bins), $
      MesSPOzone_uncert:fltarr(solar_size.num_bins), $
      MesO3QA:lonarr(solar_size.num_bins), $
      ChapO3:fltarr(solar_size.num_bins), $
      ChapO3_uncert:fltarr(solar_size.num_bins), $
      ChapSPOzone:fltarr(solar_size.num_bins), $
      ChapSPOzone_uncert:fltarr(solar_size.num_bins), $
      MLRO3QA:lonarr(solar_size.num_bins), $
      WaterVaporNDP:fltarr(solar_size.num_bins), $
      WaterVaporNDP_uncert:fltarr(solar_size.num_bins), $
      H2OQA:lonarr(solar_size.num_bins), $
      NO2NDP:fltarr(solar_size.num_bins), $
      NO2NDP_uncert:fltarr(solar_size.num_bins), $
      SPNO2:fltarr(solar_size.num_bins), $
      SPNO2_uncert:fltarr(solar_size.num_bins), $
      NO2QA:lonarr(solar_size.num_bins), $
      RetTemp:fltarr(solar_size.num_bins), $
      RetTemp_uncert:fltarr(solar_size.num_bins), $
      RetPress:fltarr(solar_size.num_bins), $
      RetPress_uncert:fltarr(solar_size.num_bins), $
      RetTPQA:lonarr(solar_size.num_bins), $
      Aer_wavelength:fltarr(solar_size.num_ao3_channels), $
      Aer_width:fltarr(solar_size.num_ao3_channels), $
      StratOD:fltarr(solar_size.num_ao3_channels), $
      StratOD_uncert:fltarr(solar_size.num_ao3_channels), $
      StratODQA:lonarr(solar_size.num_ao3_channels), $
      AerosolExt:replicate(aerosol_ext, solar_size.num_ao3_channels) , $
      asd_flag:lonarr(solar_size.num_aer_bins), $
      Aerosol1020:fltarr(solar_size.num_aer_bins), $
      Aerosol1020_uncert:fltarr(solar_size.num_aer_bins), $
      Aero1020QA:lonarr(solar_size.num_aer_bins) } 
ENDIF ELSE IF vers GT 1.06 AND vers LT 3.00 THEN BEGIN
  solar_reader_version =  save_solar_reader_version
  solar_temp = { event_id:0L, $
      yr_day_tag:0L, $
      mission_time:0., $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      L0_Version:0., $
      Software_Version:0., $
      Dataproduct_Version:0., $
      DB_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      bin_height:0., $
      num_bins:0L, $
      num_ao3_channels:0L,$
      num_grnd_trk:0L, $
      num_aer_bins:0L, $
      sc_evt_type:0L, $
      gnd_evt_type:0L, $
      BetaAngle:0., $
      st_date:0L, $
      st_time:0L, $
      st_latitude:0., $
      st_longitude:0., $
      st_altitude:0., $
      end_date:0L, $
      end_time:0L, $
      end_latitude:0., $
      end_longitude:0., $
      end_altitude:0., $
      gt_date:lonarr(solar_size.num_grnd_trk), $
      gt_time:lonarr(solar_size.num_grnd_trk), $
      gt_latitude:fltarr(solar_size.num_grnd_trk), $
      gt_longitude:fltarr(solar_size.num_grnd_trk), $
      gt_ray_dir:fltarr(solar_size.num_grnd_trk), $
      Homogeneity:lonarr(solar_size.num_bins), $
      geom_alt:fltarr(solar_size.num_bins), $
      potential_alt:fltarr(solar_size.num_bins), $
      CurrentTemp:fltarr(solar_size.num_bins), $
      CurrentTemp_uncert:fltarr(solar_size.num_bins), $
      CurrentPress:fltarr(solar_size.num_bins), $
      CurrentPress_uncert:fltarr(solar_size.num_bins), $
      TPSource:lonarr(solar_size.num_bins), $
      trop_temp:0., $
      trop_alt:0., $
      CompositeO3:fltarr(solar_size.num_bins), $
      CompositeO3_uncert:fltarr(solar_size.num_bins), $
      SPOzone:fltarr(solar_size.num_bins), $
      SPOzone_uncert:fltarr(solar_size.num_bins), $
      CompO3QA:lonarr(solar_size.num_bins), $
      MesO3:fltarr(solar_size.num_bins), $
      MesO3_uncert:fltarr(solar_size.num_bins), $
      MesSPOzone:fltarr(solar_size.num_bins), $
      MesSPOzone_uncert:fltarr(solar_size.num_bins), $
      MesO3QA:lonarr(solar_size.num_bins), $
      ChapO3:fltarr(solar_size.num_bins), $
      ChapO3_uncert:fltarr(solar_size.num_bins), $
      ChapSPOzone:fltarr(solar_size.num_bins), $
      ChapSPOzone_uncert:fltarr(solar_size.num_bins), $
      MLRO3QA:lonarr(solar_size.num_bins), $
      Ozone_aO3:fltarr(solar_size.num_bins), $
      Ozone_aO3_uncert:fltarr(solar_size.num_bins), $
      aO3SPOzone:fltarr(solar_size.num_bins), $
      aO3SPOzone_uncert:fltarr(solar_size.num_bins), $
      Ozone_aO3QA:lonarr(solar_size.num_bins), $
      WaterVaporNDP:fltarr(solar_size.num_bins), $
      WaterVaporNDP_uncert:fltarr(solar_size.num_bins), $
      H2OQA:lonarr(solar_size.num_bins), $
      NO2NDP:fltarr(solar_size.num_bins), $
      NO2NDP_uncert:fltarr(solar_size.num_bins), $
      SPNO2:fltarr(solar_size.num_bins), $
      SPNO2_uncert:fltarr(solar_size.num_bins), $
      NO2QA:lonarr(solar_size.num_bins), $
      RetTemp:fltarr(solar_size.num_bins), $
      RetTemp_uncert:fltarr(solar_size.num_bins), $
      RetPress:fltarr(solar_size.num_bins), $
      RetPress_uncert:fltarr(solar_size.num_bins), $
      RetTPQA:lonarr(solar_size.num_bins), $
      Aer_wavelength:fltarr(solar_size.num_ao3_channels), $
      Aer_width:fltarr(solar_size.num_ao3_channels), $
      StratOD:fltarr(solar_size.num_ao3_channels), $
      StratOD_uncert:fltarr(solar_size.num_ao3_channels), $
      StratODQA:lonarr(solar_size.num_ao3_channels), $
      AerosolExt:replicate(aerosol_ext, solar_size.num_ao3_channels) , $
      asd_flag:lonarr(solar_size.num_aer_bins), $
      Aerosol1020:fltarr(solar_size.num_aer_bins), $
      Aerosol1020_uncert:fltarr(solar_size.num_aer_bins), $
      Aero1020QA:lonarr(solar_size.num_aer_bins) } 
ENDIF 

;  Change file pointer to beginning of file and read in complete file
;  structure.

point_lun, in_unit, 0

if vers lt 3.00 then BEGIN
      readu, in_unit, solar_temp

      solar.event_id = solar_temp.event_id
      solar.Date = solar_temp.Date
      solar.Year_Fraction = solar_temp.Year_Fraction
      solar.Latitude = solar_temp.Latitude
      solar.Longitude = solar_temp.Longitude
      solar.Time = solar_temp.Time
      solar.int_fill_value = solar_temp.int_fill_value
      solar.flt_fill_value = solar_temp.flt_fill_value
      solar.mission_id = solar_temp.mission_id
      solar.L0DO_Version = solar_temp.L0DO_Version
      solar.CCDVersion = solar_temp.CCDVersion
      solar.L0_Version = solar_temp.L0_Version
      solar.Software_Version = solar_temp.Software_Version
      solar.Dataproduct_Version = solar_temp.Dataproduct_Version
      solar.DB_Version = solar_temp.DB_Version
      solar.GRAM95_Version = solar_temp.GRAM95_Version
      solar.Met_Version = solar_temp.Met_Version
      solar.bin_height = solar_temp.bin_height
      solar.num_bins = solar_temp.num_bins
      solar.num_ao3_channels = solar_temp.num_ao3_channels
      solar.num_grnd_trk = solar_temp.num_grnd_trk
      solar.num_aer_bins = solar_temp.num_aer_bins
      solar.sc_evt_type = solar_temp.sc_evt_type
      solar.gnd_evt_type = solar_temp.gnd_evt_type
      solar.BetaAngle = solar_temp.BetaAngle
      solar.EvtStatusFlag = 0L
      solar.st_date = solar_temp.st_date
      solar.st_time = solar_temp.st_time
      solar.st_latitude = solar_temp.st_latitude
      solar.st_longitude = solar_temp.st_longitude
      solar.st_altitude = solar_temp.st_altitude
      solar.end_date = solar_temp.end_date
      solar.end_time = solar_temp.end_time
      solar.end_latitude = solar_temp.end_latitude
      solar.end_longitude = solar_temp.end_longitude
      solar.end_altitude = solar_temp.end_altitude
      solar.gt_date = solar_temp.gt_date
      solar.gt_time = solar_temp.gt_time
      solar.gt_latitude = solar_temp.gt_latitude
      solar.gt_longitude = solar_temp.gt_longitude
      solar.gt_ray_dir = solar_temp.gt_ray_dir
      solar.Homogeneity = solar_temp.Homogeneity
      solar.geom_alt = solar_temp.geom_alt
      solar.potential_alt = solar_temp.potential_alt
      solar.CurrentTemp = solar_temp.CurrentTemp
      solar.CurrentTemp_uncert = solar_temp.CurrentTemp_uncert
      solar.CurrentPress = solar_temp.CurrentPress
      solar.CurrentPress_uncert = solar_temp.CurrentPress_uncert
      solar.TPSource = solar_temp.TPSource
      solar.trop_temp = solar_temp.trop_temp
      solar.trop_alt = solar_temp.trop_alt
      solar.CompositeO3 = solar_temp.CompositeO3
      solar.CompositeO3_uncert = solar_temp.CompositeO3_uncert
      solar.SPOzone = solar_temp.SPOzone
      solar.SPOzone_uncert = solar_temp.SPOzone_uncert
      solar.CompO3QA = solar_temp.CompO3QA
      solar.MesO3 = solar_temp.MesO3
      solar.MesO3_uncert = solar_temp.MesO3_uncert
      solar.MesO3_uncert = solar_temp.MesO3_uncert
      solar.MesSPOzone = solar_temp.MesSPOzone
      solar.MesSPOzone_uncert = solar_temp.MesSPOzone_uncert
      solar.MesO3QA = solar_temp.MesO3QA
      solar.ChapO3 = solar_temp.ChapO3
      solar.ChapO3_uncert = solar_temp.ChapO3_uncert
      solar.ChapSPOzone = solar_temp.ChapSPOzone
      solar.ChapSPOzone_uncert = solar_temp.ChapSPOzone_uncert
      solar.MLRO3QA = solar_temp.MLRO3QA
      IF vers LE 1.06 THEN BEGIN
        ; a03 was not defined before version 1.06-must be set to fill. 
        solar.Ozone_aO3(*) = solar_temp.flt_fill_value 
        solar.Ozone_aO3_uncert(*) = solar_temp.flt_fill_value
        solar.aO3SPOzone(*) =  solar_temp.flt_fill_value
        solar.aO3SPOzone_uncert(*) =  solar_temp.flt_fill_value
        solar.Ozone_aO3QA(*) = 0
      ENDIF ELSE IF vers GT 1.06 AND vers LT 3.00 THEN BEGIN
        ; a03 was defined and must must saved.
        solar.Ozone_aO3 = solar_temp.Ozone_aO3
        solar.Ozone_aO3_uncert = solar_temp.Ozone_aO3_uncert
        solar.aO3SPOzone = solar_temp.aO3SPOzone
        solar.aO3SPOzone_uncert = solar_temp.aO3SPOzone_uncert
        solar.Ozone_aO3QA = solar_temp.Ozone_aO3QA
      ENDIF
      solar.WaterVaporNDP = solar_temp.WaterVaporNDP
      solar.WaterVaporNDP_uncert = solar_temp.WaterVaporNDP_uncert
      solar.H2OQA = solar_temp.H2OQA
      solar.NO2NDP = solar_temp.NO2NDP
      solar.NO2NDP_uncert = solar_temp.NO2NDP_uncert
      solar.NO2QA = solar_temp.NO2QA
      solar.RetTemp = solar_temp.RetTemp
      solar.RetTemp_uncert = solar_temp.RetTemp_uncert
      solar.RetPress = solar_temp.RetPress
      solar.RetPress_uncert = solar_temp.RetPress_uncert
      solar.RetTPQA = solar_temp.RetTPQA
      solar.Aer_wavelength = solar_temp.Aer_wavelength
      solar.Aer_width = solar_temp.Aer_width
      solar.StratOD = solar_temp.StratOD
      solar.StratOD_uncert = solar_temp.StratOD_uncert
      solar.StratODQA = solar_temp.StratODQA
      solar.AerosolExt = solar_temp.AerosolExt
      solar.Aerosol1020 = solar_temp.Aerosol1020
      solar.Aerosol1020_uncert = solar_temp.Aerosol1020_uncert
      solar.Aero1020QA = solar_temp.Aero1020QA
     if IEEE_flag eq 0 then solar = swap_endian(solar)
ENDIF ELSE IF vers GE 3 and vers LE 5.0 THEN BEGIN 
 
   readu, in_unit, solar
   if IEEE_flag eq 0 then solar = swap_endian(solar)

   save_reader_temp =  solar.EvtStatusFlag
   save_reader_templ =  ishft(save_reader_temp,24)
   solar_reader_version =  ishft(save_reader_templ,-27)
ENDIF

IF solar_reader_version LT save_solar_reader_version THEN BEGIN 
  status =  'NEW READER'
  print, '****************************************************************'
  print, ' Product Format has changed -- you MUST get new reader software' 
  print, '****************************************************************'
  stop
ENDIF 

;  Close data file unit and free the logical unit number.

close, in_unit
free_lun, in_unit
goto, program_end

open_error:
print, 'Error on open or read of file ', filename
status = 'ERROR'

program_end:
end
