pro read_solar_trans, filename, status, solar_trans

;Disclaimers
;No Warranty: THE SUBJECT SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY OF
;ANY KIND, EITHER EXPRESSED, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT LIMITED
;TO, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL CONFORM TO SPECIFICATIONS,
;ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
;OR FREEDOM FROM INFRINGEMENT, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL BE
;ERROR FREE, OR ANY WARRANTY THAT DOCUMENTATION, IF PROVIDED, WILL CONFORM TO
;THE SUBJECT SOFTWARE. THIS AGREEMENT DOES NOT, IN ANY MANNER, CONSTITUTE AN
;ENDORSEMENT BY GOVERNMENT AGENCY OR ANY PRIOR RECIPIENT OF ANY RESULTS,
;RESULTING DESIGNS, HARDWARE, SOFTWARE PRODUCTS OR ANY OTHER APPLICATIONS
;RESULTING FROM USE OF THE SUBJECT SOFTWARE.  FURTHER, GOVERNMENT AGENCY
;DISCLAIMS ALL WARRANTIES AND LIABILITIES REGARDING THIRD-PARTY SOFTWARE, IF
;PRESENT IN THE ORIGINAL SOFTWARE, AND DISTRIBUTES IT "AS IS." 
 
;Waiver and Indemnity:  RECIPIENT AGREES TO WAIVE ANY AND ALL CLAIMS AGAINST
;THE UNITED STATES GOVERNMENT, ITS CONTRACTORS AND SUBCONTRACTORS, AS WELL AS
;ANY PRIOR RECIPIENT.  IF RECIPIENT'S USE OF THE SUBJECT SOFTWARE RESULTS IN
;ANY LIABILITIES, DEMANDS, DAMAGES, EXPENSES OR LOSSES ARISING FROM SUCH USE,
;INCLUDING ANY DAMAGES FROM PRODUCTS BASED ON, OR RESULTING FROM, RECIPIENT'S
;USE OF THE SUBJECT SOFTWARE, RECIPIENT SHALL INDEMNIFY AND HOLD HARMLESS THE
;UNITED STATES GOVERNMENT, ITS CONTRACTORS AND SUBCONTRACTORS, AS WELL AS ANY
;PRIOR RECIPIENT, TO THE EXTENT PERMITTED BY LAW.  RECIPIENT'S SOLE REMEDY FOR
;ANY SUCH MATTER SHALL BE THE IMMEDIATE, UNILATERAL TERMINATION OF THIS
;AGREEMENT.

;  Purpose:
;  This program provides the data structures necessary to read the
;  SAGE III Solar Transmission binary product files.
;
;
;  Author:
;  Mike Heitz <michael.l.heitz@nasa.gov>
;  Carter Hulsey
;  Robbie Manion
;  Marsha Larosee
;  David Huber
;  Karen H. Sage
;  
;  Revision History:
;  11/01/2000 - Initial Coding.
;  10/16/02 - Changed solar_trans to an anonymous structure to give
;             more flexibility.
;  03/12/03 - Renamed Director Version and Solar Version to 
;             Software Version and Dataproduct Version.
;  09/19/03 - Added new variable (EvtStatusFlag) to solar_trans structure.  
;             Made code backward compatible.
;  11/18/03 - Set vers variable.
;  09/19/18 - Added compatability with version 5.1
;
;  Input variable:  the name of the solar transmission product file
;     (the product file names have the form  g3a.tb.*)
;  Output variables:  a read file status variable and the structure
;     containing the solar transmission product data.  See the SAGE III
;     Data Products Users Guide for a description of the solar
;     transmission product data.

;  Initialize local variables.

status = 'OK'
IEEE_flag = 0

itest = 512
ltest = 102580L
ftest = 1.23e10
dtest = 1.23d50


;  Determine if the current machine uses IEEE, big_endian numbers.
;    A sample int, long, float and double are converted using
;    byteorder and compared with the original.  If there is no 
;    change, the machine is assumed to be IEEE compliant and big-endian.
;    

it2 = itest
lt2 = ltest
ft2 = ftest
dt2 = dtest

byteorder, it2, /htons
byteorder, lt2, /htonl
byteorder, ft2, /ftoxdr
byteorder, dt2, /dtoxdr

case 1 of
(itest eq it2) and (ltest eq lt2) and (ftest eq ft2) and (dtest eq dt2) : IEEE_flag = 1
else:  IEEE_flag = 0
endcase

;Get Version from file name 
splt_file = STRSPLIT(filename, 'v', /EXTRACT)
vers = FLOAT(splt_file[-1])

;  Create the structure for read from binary file of the data needed
;  to dynamically size data arrays.

IF vers GE 5.3 THEN BEGIN
        solar_trans_size = {event_id:'XXXXXXXXXXXX', $
                         old_event_id:0L, $
                         Date:0L, $
                         fraction_time:0.D, $
                         Latitude:0L, $
                         Longitude:0L, $
                         Time:0L, $
                         int_fill_value:0L, $
                         flt_fill_value:0., $
                         mission_id:0L, $
                         L0DO_Version:0., $
                         CCDVersion:0., $
                         L0_Version:0., $
                         Software_Version:0., $
                         Dataproduct_Version:0., $
                         Spectroscopic_DataBase_Version:0., $
                         GRAM95_Version:0., $
                         Met_Version:0., $
                         bin_height:0., $
                         profile_count:0L, $
                         num_grnd_trk:0L, $
                         num_press_grid:0L, $
                         num_CCDPxlGrps:0L, $
                         num_alt_bins:0L }

ENDIF ELSE IF vers GE 5.2 THEN BEGIN
        solar_trans_size = {event_id:'XXXXXXXXXXXX', $
                         old_event_id:0L, $
                         Date:0L, $
                         fraction_time:0., $
                         Latitude:0L, $
                         Longitude:0L, $
                         Time:0L, $
                         int_fill_value:0L, $
                         flt_fill_value:0., $
                         mission_id:0L, $
                         L0DO_Version:0., $
                         CCDVersion:0., $
                         L0_Version:0., $
                         Software_Version:0., $
                         Dataproduct_Version:0., $
                         Spectroscopic_DataBase_Version:0., $
                         GRAM95_Version:0., $
                         Met_Version:0., $
                         bin_height:0., $
                         profile_count:0L, $
                         num_grnd_trk:0L, $
                         num_press_grid:0L, $
                         num_CCDPxlGrps:0L, $
                         num_alt_bins:0L }


ENDIF ELSE IF vers EQ 5.1 THEN BEGIN
	solar_trans_size = {event_id:0L, $
                         Date:0L, $
                         fraction_time:0., $
                         Latitude:0L, $
                         Longitude:0L, $
                         Time:0L, $
                         int_fill_value:0L, $
                         flt_fill_value:0., $
                         mission_id:0L, $
                         L0DO_Version:0., $
                         CCDVersion:0., $
                         L0_Version:0., $
                         Software_Version:0., $
                         Dataproduct_Version:0., $
                         Spectroscopic_DataBase_Version:0., $
                         GRAM95_Version:0., $
                         Met_Version:0., $
                         bin_height:0., $
                         profile_count:0L, $
                         num_grnd_trk:0L, $
                         num_press_grid:0L, $
                         num_CCDPxlGrps:0L, $
                         num_alt_bins:0L }
ENDIF ELSE BEGIN
	solar_trans_size = {event_id:0L, $
	                 yr_day_tag:0L, $
	                 mission_time:0., $
	                 int_fill_value:0L, $
	                 flt_fill_value:0., $
	                 mission_id:0L, $
	                 L0DO_Version:0., $
	                 L0_Version:0., $
	                 Software_Version:0., $
	                 Dataproduct_Version:0., $
	                 DB_Version:0., $
	                 GRAM95_Version:0., $
	                 Met_Version:0., $
	                 bin_height:0., $
	                 profile_count:0L, $
	                 num_grnd_trk:0L, $
	                 num_press_grid:0L, $
	                 num_CCDPxlGrps:0L, $
	                 num_alt_bins:0L }
ENDELSE

;  Open file for direct read-only access.

on_ioerror, open_error
openr, in_unit, filename, /get_lun

;  Read size structure.

readu, in_unit, solar_trans_size

;  If machine is not IEEE then swap bytes to make them compliant.
 
if IEEE_flag eq 0 then solar_trans_size = swap_endian (solar_trans_size)



;  Create a structure for the pixel group transmission profile.
;  NOTE:  The tag pg_trans_profiles is an array containing this structure
;         for each channel.  To reference the i-th element of the
;         array, an example of the correct syntax is 
;         solar_trans.pg_trans_profiles(i).Transmission

pg_dmp_profile = { dmp_profile, $
                      Magnitude:fltarr(solar_trans_size.num_alt_bins), $
                      Fraction:fltarr(solar_trans_size.num_alt_bins) }

;  Create a structure for the pixel group transmission profile.
;  NOTE:  The tag pg_trans_profiles is an array containing this structure
;         for each channel.  To reference the i-th element of the
;         array, an example of the correct syntax is 
;         solar_trans.pg_trans_profiles(i).Transmission

pg_trans_profile = { trans_profile, $
                      Transmission:fltarr(solar_trans_size.num_alt_bins,2), $
                      TransQA:lonarr(solar_trans_size.num_alt_bins) }

 
IF vers GE 5.3 THEN BEGIN
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;  Define the solar trans structure for version 5.3 ;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  solar_trans = { event_id:'XXXXXXXXXXXX', $
      old_event_id:0L, $
      Date:0L, $
      Year_Fraction:0.D, $
      Latitude:0., $
      Longitude:0., $
      Time:0L, $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      CCDVersion:0L, $
      L0_Version:0., $
      Software_Version:0., $
      Dataproduct_Version:0., $
      Spectroscopic_DataBase_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      bin_height:0., $
      profile_count:0L, $
      num_grnd_trk:0L, $
      num_press_grid:0L, $
      num_CCDPxlGrps:0L, $
      num_alt_bins:0L, $
      sc_evt_type:0L, $
      gnd_evt_type:0L, $
      BetaAngle_Solar:0., $
      Aurora_Flag:0L, $
      Ephemeris_Source:0L, $
      gt_date:lonarr(solar_trans_size.num_grnd_trk), $
      gt_time:lonarr(solar_trans_size.num_grnd_trk), $
      gt_latitude:fltarr(solar_trans_size.num_grnd_trk), $
      gt_longitude:fltarr(solar_trans_size.num_grnd_trk), $
      gt_ray_dir:fltarr(solar_trans_size.num_grnd_trk), $
      Space_Craft_Lat:fltarr(solar_trans_size.num_grnd_trk), $
      Space_Craft_Lon:fltarr(solar_trans_size.num_grnd_trk), $
      Space_Craft_Alt:fltarr(solar_trans_size.num_grnd_trk), $
      Altitude:fltarr(solar_trans_size.num_alt_bins), $
      Geopotential_Alt:fltarr(solar_trans_size.num_alt_bins), $
      Pressure:fltarr(solar_trans_size.num_alt_bins), $
      Pressure_uncert:fltarr(solar_trans_size.num_alt_bins), $
      Temperature:fltarr(solar_trans_size.num_alt_bins), $
      Temperature_uncert:fltarr(solar_trans_size.num_alt_bins), $
      Neutral_Density:fltarr(solar_trans_size.num_alt_bins), $
      Neutral_Density_uncert:fltarr(solar_trans_size.num_alt_bins), $
      Temp_Pressure_Source:lonarr(solar_trans_size.num_alt_bins), $
      trop_temp:0., $
      trop_alt:0., $
      trop_press:0., $
      met_pressure:fltarr(solar_trans_size.num_press_grid), $
      met_temp:fltarr(solar_trans_size.num_press_grid), $
      met_temp_unc:fltarr(solar_trans_size.num_press_grid), $
      met_altitude:fltarr(solar_trans_size.num_press_grid), $
      met_Source:0L, $
      CCD_Temperature:0., $
      Spectrometer_Zenith_Temperature:0., $
      CCD_Temperature_minus_TEC:0., $
      Ephemeris_Quality:0L, $
      SpecCalShift:0.,$
      SpecCalStretch:0.,$
      AzimuthAngle:fltarr(2), $
      QAFlag:0L, $
      QAFlag_Altitude:lonarr(solar_trans_size.num_alt_bins), $
      start_pixel_num:lonarr(solar_trans_size.num_CCDPxlGrps), $
      end_pixel_num:lonarr(solar_trans_size.num_CCDPxlGrps), $
      central_wavelength:fltarr(solar_trans_size.profile_count), $
      half_bandwidth:fltarr(solar_trans_size.profile_count), $
      pg_dmp_profiles:replicate( pg_dmp_profile , 87 ), $
      pg_trans_profiles:replicate( pg_trans_profile , 87 ) }

  point_lun, in_unit, 0
  readu, in_unit, solar_trans

  ;  If machine is not IEEE then swap bytes to make them compliant.
  if IEEE_flag eq 0 then solar_trans = swap_endian(solar_trans)

  ;Add Separate Flags for Event QA BitFlag Entries, IDL 1/0 = True/False
  solar_trans_addtags={HexErrFlag:0L,ContWindowClosedFlag:0L,TimeQualFlag:0L,DMPExoFlag:0L,BlockExoFlag:0L,$
        SpectralCalFlag:0L,SolarEclipseFlag:0L,OffNadirFlag:0L,DMPSkipFlag:0L,DMPAltFlag:lonarr(solar_trans_size.num_alt_bins)}

  solar_trans=create_struct(solar_trans,solar_trans_addtags)

  if ((solar_trans.QAFlag and 2L^0) ne 0) then solar_trans.HexErrFlag=1 else solar_trans.HexErrFlag=0
  if ((solar_trans.QAFlag and 2L^1) ne 0) then solar_trans.ContWindowClosedFlag=1 else solar_trans.ContWindowClosedFlag=0
  if ((solar_trans.QAFlag and 2L^2) ne 0) then solar_trans.TimeQualFlag=1 else solar_trans.TimeQualFlag=0
  if ((solar_trans.QAFlag and 2L^3) ne 0) then solar_trans.DMPExoFlag=1 else solar_trans.DMPExoFlag=0
  if ((solar_trans.QAFlag and 2L^4) ne 0) then solar_trans.BlockExoFlag=1 else solar_trans.BlockExoFlag=0
  if ((solar_trans.QAFlag and 2L^5) ne 0) then solar_trans.SpectralCalFlag=1 else solar_trans.SpectralCalFlag=0
  if ((solar_trans.QAFlag and 2L^6) ne 0) then solar_trans.SolarEclipseFlag=1 else solar_trans.SolarEclipseFlag=0
  if ((solar_trans.QAFlag and 2L^7) ne 0) then solar_trans.OffNadirFlag=1 else solar_trans.OffNadirFlag=0
  if ((solar_trans.QAFlag and 2L^8) ne 0) then solar_trans.DMPSkipFlag=1 else solar_trans.DMPSkipFlag=0

  ;Altitude Based Bit Flags
  for i = 0,solar_trans_size.num_alt_bins-1 do begin
      if ((solar_trans.QAFlag_Altitude(i) and 2^0) ne 0) then solar_trans.DMPAltFlag(i)=1 else solar_trans.DMPAltFlag(i)=0
  endfor

  close, in_unit
  free_lun, in_unit
  goto, program_end


ENDIF ELSE IF vers EQ 5.2 THEN BEGIN
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;  Define the solar trans structure for version 5.2 ;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  solar_trans = { event_id:'XXXXXXXXXXXX', $
      old_event_id:0L, $
      Date:0L, $
      Year_Fraction:0., $
      Latitude:0., $
      Longitude:0., $
      Time:0L, $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      CCDVersion:0L, $
      L0_Version:0., $
      Software_Version:0., $
      Dataproduct_Version:0., $
      Spectroscopic_DataBase_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      bin_height:0., $
      profile_count:0L, $
      num_grnd_trk:0L, $
      num_press_grid:0L, $
      num_CCDPxlGrps:0L, $
      num_alt_bins:0L, $
      sc_evt_type:0L, $
      gnd_evt_type:0L, $
      BetaAngle_Solar:0., $
      Aurora_Flag:0L, $
      Ephemeris_Source:0L, $
      gt_date:lonarr(solar_trans_size.num_grnd_trk), $
      gt_time:lonarr(solar_trans_size.num_grnd_trk), $
      gt_latitude:fltarr(solar_trans_size.num_grnd_trk), $
      gt_longitude:fltarr(solar_trans_size.num_grnd_trk), $
      gt_ray_dir:fltarr(solar_trans_size.num_grnd_trk), $
      Space_Craft_Lat:fltarr(solar_trans_size.num_grnd_trk), $
      Space_Craft_Lon:fltarr(solar_trans_size.num_grnd_trk), $
      Space_Craft_Alt:fltarr(solar_trans_size.num_grnd_trk), $
      Altitude:fltarr(solar_trans_size.num_alt_bins), $
      Geopotential_Alt:fltarr(solar_trans_size.num_alt_bins), $
      Pressure:fltarr(solar_trans_size.num_alt_bins), $
      Pressure_uncert:fltarr(solar_trans_size.num_alt_bins), $
      Temperature:fltarr(solar_trans_size.num_alt_bins), $
      Temperature_uncert:fltarr(solar_trans_size.num_alt_bins), $
      Neutral_Density:fltarr(solar_trans_size.num_alt_bins), $
      Neutral_Density_uncert:fltarr(solar_trans_size.num_alt_bins), $
      Temp_Pressure_Source:lonarr(solar_trans_size.num_alt_bins), $
      trop_temp:0., $
      trop_alt:0., $
      trop_press:0., $
      met_pressure:fltarr(solar_trans_size.num_press_grid), $
      met_temp:fltarr(solar_trans_size.num_press_grid), $
      met_temp_unc:fltarr(solar_trans_size.num_press_grid), $
      met_altitude:fltarr(solar_trans_size.num_press_grid), $
      met_Source:0L, $
      CCD_Temperature:0., $
      Spectrometer_Zenith_Temperature:0., $
      CCD_Temperature_minus_TEC:0., $
      Ephemeris_Quality:0L, $
      SpecCalShift:0.,$
      SpecCalStretch:0.,$
      AzimuthAngle:fltarr(2), $
      QAFlag:0L, $
      QAFlag_Altitude:lonarr(solar_trans_size.num_alt_bins), $
      start_pixel_num:lonarr(solar_trans_size.num_CCDPxlGrps), $
      end_pixel_num:lonarr(solar_trans_size.num_CCDPxlGrps), $
      central_wavelength:fltarr(solar_trans_size.profile_count), $
      half_bandwidth:fltarr(solar_trans_size.profile_count), $
      pg_trans_profiles:replicate( pg_trans_profile , 87 ) }

  point_lun, in_unit, 0
  readu, in_unit, solar_trans

  ;  If machine is not IEEE then swap bytes to make them compliant.
  if IEEE_flag eq 0 then solar_trans = swap_endian(solar_trans)

  ;Add Separate Flags for Event QA BitFlag Entries, IDL 1/0 = True/False
  solar_trans_addtags={HexErrFlag:0L,ContWindowClosedFlag:0L,TimeQualFlag:0L,DMPExoFlag:0L,BlockExoFlag:0L,$
        SpectralCalFlag:0L,SolarEclipseFlag:0L, DMPAltFlag:lonarr(solar_trans_size.num_alt_bins)}

  solar_trans=create_struct(solar_trans,solar_trans_addtags)

  if ((solar_trans.QAFlag and 2L^0) ne 0) then solar_trans.HexErrFlag=1 else solar_trans.HexErrFlag=0
  if ((solar_trans.QAFlag and 2L^1) ne 0) then solar_trans.ContWindowClosedFlag=1 else solar_trans.ContWindowClosedFlag=0
  if ((solar_trans.QAFlag and 2L^2) ne 0) then solar_trans.TimeQualFlag=1 else solar_trans.TimeQualFlag=0
  if ((solar_trans.QAFlag and 2L^3) ne 0) then solar_trans.DMPExoFlag=1 else solar_trans.DMPExoFlag=0
  if ((solar_trans.QAFlag and 2L^4) ne 0) then solar_trans.BlockExoFlag=1 else solar_trans.BlockExoFlag=0
  if ((solar_trans.QAFlag and 2L^5) ne 0) then solar_trans.SpectralCalFlag=1 else solar_trans.SpectralCalFlag=0
  if ((solar_trans.QAFlag and 2L^6) ne 0) then solar_trans.SolarEclipseFlag=1 else solar_trans.SolarEclipseFlag=0

  ;Altitude Based Bit Flags
  for i = 0,solar_trans_size.num_alt_bins-1 do begin
      if ((solar_trans.QAFlag_Altitude(i) and 2^0) ne 0) then solar_trans.DMPAltFlag(i)=1 else solar_trans.DMPAltFlag(i)=0
  endfor

  close, in_unit
  free_lun, in_unit
  goto, program_end


ENDIF ELSE IF vers EQ 5.1 THEN BEGIN
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;  Define the solar trans structure for version 5.1 ;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  solar_trans = { event_id:0L, $
      Date:0L, $
      Year_Fraction:0., $
      Latitude:0., $
      Longitude:0., $
      Time:0L, $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      CCDVersion:0L, $
      L0_Version:0., $
      Software_Version:0., $
      Dataproduct_Version:0., $
      Spectroscopic_DataBase_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      bin_height:0., $
      profile_count:0L, $
      num_grnd_trk:0L, $
      num_press_grid:0L, $
      num_CCDPxlGrps:0L, $
      num_alt_bins:0L, $
      sc_evt_type:0L, $
      gnd_evt_type:0L, $
      BetaAngle_Solar:0., $
      Aurora_Flag:0L, $
      Ephemeris_Source:0L, $
      gt_date:lonarr(solar_trans_size.num_grnd_trk), $
      gt_time:lonarr(solar_trans_size.num_grnd_trk), $
      gt_latitude:fltarr(solar_trans_size.num_grnd_trk), $
      gt_longitude:fltarr(solar_trans_size.num_grnd_trk), $
      gt_ray_dir:fltarr(solar_trans_size.num_grnd_trk), $
      Space_Craft_Lat:fltarr(solar_trans_size.num_grnd_trk), $
      Space_Craft_Lon:fltarr(solar_trans_size.num_grnd_trk), $
      Space_Craft_Alt:fltarr(solar_trans_size.num_grnd_trk), $
      Altitude:fltarr(solar_trans_size.num_alt_bins), $
      Geopotential_Alt:fltarr(solar_trans_size.num_alt_bins), $
      Pressure:fltarr(solar_trans_size.num_alt_bins), $
      Pressure_uncert:fltarr(solar_trans_size.num_alt_bins), $
      Temperature:fltarr(solar_trans_size.num_alt_bins), $
      Temperature_uncert:fltarr(solar_trans_size.num_alt_bins), $
      Neutral_Density:fltarr(solar_trans_size.num_alt_bins), $
      Neutral_Density_uncert:fltarr(solar_trans_size.num_alt_bins), $
      Temp_Pressure_Source:lonarr(solar_trans_size.num_alt_bins), $
      trop_temp:0., $
      trop_alt:0., $
      trop_press:0., $
      met_pressure:fltarr(solar_trans_size.num_press_grid), $
      met_temp:fltarr(solar_trans_size.num_press_grid), $
      met_temp_unc:fltarr(solar_trans_size.num_press_grid), $
      met_altitude:fltarr(solar_trans_size.num_press_grid), $
      met_Source:0L, $
      CCD_Temperature:0., $
      Spectrometer_Zenith_Temperature:0., $
      CCD_Temperature_minus_TEC:0., $
      Ephemeris_Quality:0L, $
      SpecCalShift:0.,$
      SpecCalStretch:0.,$
      QAFlag:0L, $
      QAFlag_Altitude:lonarr(solar_trans_size.num_alt_bins), $
      start_pixel_num:lonarr(solar_trans_size.num_CCDPxlGrps), $
      end_pixel_num:lonarr(solar_trans_size.num_CCDPxlGrps), $
      central_wavelength:fltarr(solar_trans_size.num_CCDPxlGrps), $
      half_bandwidth:fltarr(solar_trans_size.num_CCDPxlGrps), $
      pg_trans_profiles:replicate( pg_trans_profile , 87 ) }

  point_lun, in_unit, 0
  readu, in_unit, solar_trans

  ;  If machine is not IEEE then swap bytes to make them compliant.
  if IEEE_flag eq 0 then solar_trans = swap_endian(solar_trans)

  ;Add Separate Flags for Event QA BitFlag Entries, IDL 1/0 = True/False
  solar_trans_addtags={HexErrFlag:0L,ContWindowClosedFlag:0L,TimeQualFlag:0L,DMPExoFlag:0L,BlockExoFlag:0L,$
        SpectralCalFlag:0L,SolarEclipseFlag:0L, DMPAltFlag:lonarr(solar_trans_size.num_alt_bins)}

  solar_trans=create_struct(solar_trans,solar_trans_addtags)

  if ((solar_trans.QAFlag and 2L^0) ne 0) then solar_trans.HexErrFlag=1 else solar_trans.HexErrFlag=0
  if ((solar_trans.QAFlag and 2L^1) ne 0) then solar_trans.ContWindowClosedFlag=1 else solar_trans.ContWindowClosedFlag=0
  if ((solar_trans.QAFlag and 2L^2) ne 0) then solar_trans.TimeQualFlag=1 else solar_trans.TimeQualFlag=0
  if ((solar_trans.QAFlag and 2L^3) ne 0) then solar_trans.DMPExoFlag=1 else solar_trans.DMPExoFlag=0
  if ((solar_trans.QAFlag and 2L^4) ne 0) then solar_trans.BlockExoFlag=1 else solar_trans.BlockExoFlag=0
  if ((solar_trans.QAFlag and 2L^5) ne 0) then solar_trans.SpectralCalFlag=1 else solar_trans.SpectralCalFlag=0
  if ((solar_trans.QAFlag and 2L^6) ne 0) then solar_trans.SolarEclipseFlag=1 else solar_trans.SolarEclipseFlag=0

  ;Altitude Based Bit Flags
  for i = 0,solar_trans_size.num_alt_bins-1 do begin
      if ((solar_trans.QAFlag_Altitude(i) and 2^0) ne 0) then solar_trans.DMPAltFlag(i)=1 else solar_trans.DMPAltFlag(i)=0
  endfor

  close, in_unit
  free_lun, in_unit
  goto, program_end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;; The Following Structures are for reading older dicontinued file version. Current Data Product ;;;;
;;;;;                      version (5.1 and greater) us the structure above                         ;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;  Create the structure for read of all data from binary file.
ENDIF ELSE if vers lt 3.00 then BEGIN
  solar_trans_temp = { event_id:0L, $
      yr_day_tag:0L, $
      mission_time:0., $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      L0_Version:0., $
      Software_Version:0., $
      Dataproduct_Version:0., $
      DB_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      bin_height:0., $
      profile_count:0L, $
      num_grnd_trk:0L, $
      num_press_grid:0L, $
      num_CCDPxlGrps:0L, $
      num_alt_bins:0L, $
      sc_evt_type:0L, $
      gnd_evt_type:0L, $
      BetaAngle:0., $
      st_date:0L, $
      st_time:0L, $
      st_latitude:0., $
      st_longitude:0., $
      st_altitude:0., $
      end_date:0L, $
      end_time:0L, $
      end_latitude:0., $
      end_longitude:0., $
      end_altitude:0., $
      gt_date:lonarr(solar_trans_size.num_grnd_trk), $
      gt_time:lonarr(solar_trans_size.num_grnd_trk), $
      gt_latitude:fltarr(solar_trans_size.num_grnd_trk), $
      gt_longitude:fltarr(solar_trans_size.num_grnd_trk), $
      gt_ray_dir:fltarr(solar_trans_size.num_grnd_trk), $
      trop_temp:0., $
      trop_alt:0., $
      met_pressure:fltarr(solar_trans_size.num_press_grid), $
      met_temp:fltarr(solar_trans_size.num_press_grid), $
      met_temp_unc:fltarr(solar_trans_size.num_press_grid), $
      met_altitude:fltarr(solar_trans_size.num_press_grid), $
      met_Source:0L, $
      start_pixel_num:lonarr(solar_trans_size.num_CCDPxlGrps), $
      end_pixel_num:lonarr(solar_trans_size.num_CCDPxlGrps), $
      central_wavelength:fltarr(solar_trans_size.num_CCDPxlGrps), $
      half_bandwidth:fltarr(solar_trans_size.num_CCDPxlGrps), $
      Altitude:fltarr(solar_trans_size.num_alt_bins), $
      CurrentPress:fltarr(solar_trans_size.num_alt_bins), $
      CurrentTemp:fltarr(solar_trans_size.num_alt_bins), $
      CurrentTemp_uncert:fltarr(solar_trans_size.num_alt_bins), $
      TPSource:lonarr(solar_trans_size.num_alt_bins), $ 
      pg_trans_profiles:replicate( pg_trans_profile , 87 ) }

;  Create the structure for read of all data from binary file. 
;  NOTE:  Last tag in the structure is an array containing the 
;   pg_trans_profile structure.

ENDIF ELSE IF vers GE 3.0 AND vers LE 5.0 THEN BEGIN 
  solar_trans = { event_id:0L, $
      yr_day_tag:0L, $
      mission_time:0., $
      int_fill_value:0L, $
      flt_fill_value:0., $
      mission_id:0L, $
      L0DO_Version:0., $
      L0_Version:0., $
      Software_Version:0., $
      Dataproduct_Version:0., $
      DB_Version:0., $
      GRAM95_Version:0., $
      Met_Version:0., $
      bin_height:0., $
      profile_count:0L, $
      num_grnd_trk:0L, $
      num_press_grid:0L, $
      num_CCDPxlGrps:0L, $
      num_alt_bins:0L, $
      sc_evt_type:0L, $
      gnd_evt_type:0L, $
      BetaAngle:0., $
      EvtStatusFlag:0L, $
      st_date:0L, $
      st_time:0L, $
      st_latitude:0., $
      st_longitude:0., $
      st_altitude:0., $
      end_date:0L, $
      end_time:0L, $
      end_latitude:0., $
      end_longitude:0., $
      end_altitude:0., $
      gt_date:lonarr(solar_trans_size.num_grnd_trk), $
      gt_time:lonarr(solar_trans_size.num_grnd_trk), $
      gt_latitude:fltarr(solar_trans_size.num_grnd_trk), $
      gt_longitude:fltarr(solar_trans_size.num_grnd_trk), $
      gt_ray_dir:fltarr(solar_trans_size.num_grnd_trk), $
      trop_temp:0., $
      trop_alt:0., $
      met_pressure:fltarr(solar_trans_size.num_press_grid), $
      met_temp:fltarr(solar_trans_size.num_press_grid), $
      met_temp_unc:fltarr(solar_trans_size.num_press_grid), $
      met_altitude:fltarr(solar_trans_size.num_press_grid), $
      met_Source:0L, $
      start_pixel_num:lonarr(solar_trans_size.num_CCDPxlGrps), $
      end_pixel_num:lonarr(solar_trans_size.num_CCDPxlGrps), $
      central_wavelength:fltarr(solar_trans_size.num_CCDPxlGrps), $
      half_bandwidth:fltarr(solar_trans_size.num_CCDPxlGrps), $
      Altitude:fltarr(solar_trans_size.num_alt_bins), $
      CurrentPress:fltarr(solar_trans_size.num_alt_bins), $
      CurrentTemp:fltarr(solar_trans_size.num_alt_bins), $
      CurrentTemp_uncert:fltarr(solar_trans_size.num_alt_bins), $
      TPSource:lonarr(solar_trans_size.num_alt_bins), $
      pg_trans_profiles:replicate( pg_trans_profile , 87 ) }
ENDIF
;  Change file pointer to beginning of file and read in complete file
;  structure.

point_lun, in_unit, 0

if vers lt 3.00 then readu, in_unit, solar_trans_temp $
ELSE readu, in_unit, solar_trans

if vers lt 3.00 then BEGIN
     solar_trans.Event_ID = solar_trans_temp.Event_ID
     solar_trans.yr_day_tag = solar_trans_temp.yr_day_tag 
     solar_trans.mission_time = solar_trans_temp.mission_time
     solar_trans.int_fill_value = solar_trans_temp.int_fill_value
     solar_trans.flt_fill_value = solar_trans_temp.flt_fill_value
     solar_trans.mission_id = solar_trans_temp.mission_id
     solar_trans.L0DO_Version = solar_trans_temp.L0DO_Version 
     solar_trans.L0_Version = solar_trans_temp.L0_Version
     solar_trans.Software_Version = solar_trans_temp.Software_Version
     solar_trans.Dataproduct_Version = solar_trans_temp.Dataproduct_Version
     solar_trans.DB_Version = solar_trans_temp.DB_Version 
     solar_trans.GRAM95_Version = solar_trans_temp.GRAM95_Version
     solar_trans.Met_Version = solar_trans_temp.Met_Version 
     solar_trans.bin_height = solar_trans_temp.bin_height
     solar_trans.profile_count = solar_trans_temp.profile_count
     solar_trans.num_grnd_trk = solar_trans_temp.num_grnd_trk
     solar_trans.num_press_grid = solar_trans_temp.num_press_grid 
     solar_trans.num_CCDPxlGrps = solar_trans_temp.num_CCDPxlGrps
     solar_trans.num_alt_bins = solar_trans_temp.num_alt_bins
     solar_trans.sc_evt_type = solar_trans_temp.sc_evt_type
     solar_trans.gnd_evt_type = solar_trans_temp.gnd_evt_type
     solar_trans.BetaAngle = solar_trans_temp.BetaAngle
     solar_trans.EvtStatusFlag = solar_trans_temp.int_fill_value
     solar_trans.st_date = solar_trans_temp.st_date
     solar_trans.st_time = solar_trans_temp.st_time
     solar_trans.st_latitude = solar_trans_temp.st_latitude
     solar_trans.st_longitude = solar_trans_temp.st_longitude
     solar_trans.st_altitude = solar_trans_temp.st_altitude
     solar_trans.end_time = solar_trans_temp.end_time
     solar_trans.end_latitude = solar_trans_temp.end_latitude
     solar_trans.end_longitude = solar_trans_temp.end_longitude
     solar_trans.end_altitude = solar_trans_temp.end_altitude
     solar_trans.gt_date = solar_trans_temp.gt_date
     solar_trans.gt_time = solar_trans_temp.gt_time
     solar_trans.gt_latitude = solar_trans_temp.gt_latitude
     solar_trans.gt_longitude = solar_trans_temp.gt_longitude
     solar_trans.gt_ray_dir = solar_trans_temp.gt_ray_dir
     solar_trans.trop_temp = solar_trans_temp.trop_temp
     solar_trans.trop_alt = solar_trans_temp.trop_alt 
     solar_trans.met_pressure = solar_trans_temp.met_pressure
     solar_trans.met_temp = solar_trans_temp.met_temp
     solar_trans.met_temp_unc = solar_trans_temp.met_temp_unc
     solar_trans.met_altitude = solar_trans_temp.met_altitude
     solar_trans.Met_Source = solar_trans_temp.Met_Source
     solar_trans.start_pixel_num = solar_trans_temp.start_pixel_num
     solar_trans.end_pixel_num = solar_trans_temp.end_pixel_num
     solar_trans.central_wavelength = solar_trans_temp.central_wavelength
     solar_trans.half_bandwidth = solar_trans_temp.half_bandwidth
     solar_trans.Altitude = solar_trans_temp.Altitude
     solar_trans.CurrentPress = solar_trans_temp.CurrentPress
     solar_trans.CurrentTemp = solar_trans_temp.CurrentTemp
     solar_trans.CurrentTemp_uncert = solar_trans_temp.CurrentTemp_uncert
     solar_trans.TPSource = solar_trans_temp.TPSource
     solar_trans.pg_trans_profiles = solar_trans_temp.pg_trans_profiles
ENDIF

 ;  If machine is not IEEE then swap bytes to make them compliant.
 
if IEEE_flag eq 0 then solar_trans = swap_endian(solar_trans)

close, in_unit
free_lun, in_unit

goto, program_end

open_error:
print, 'Error on open or read of file ', filename
status = 'ERROR'

program_end:
end


